

-module(link3).
-export([start/0, child_process/0]).

% shows 2-ways links, here the child process ends abnormally
% before the parent, and the parent  gets that non=normal exit
% signal.  Here we see default behavior for non-normal exit
% in linked processes.
% the parent ends as well, ungraciously

start() ->
    % Parent process continues running after child finishes
    io:format("Parent>> process begins running.~n"),

    % Parent process spawns and links to the child process
    ChildPid = spawn_link(?MODULE, child_process, []),
    io:format("Parent>> process has spawned and linked to child.~n"),


    % Wait for a while before ending
    timer:sleep(2000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(2000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(2000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(2000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(2000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(2000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(2000),
    io:format("Parent>> process exiting normally.~n").

child_process() ->
    io:format("Child>> process is running.~n"),
    timer:sleep(3000),
    io:format("Child>> process is running.~n"),
    timer:sleep(3000),
    io:format("Child>> process is running.~n"),

    % Child process finishes abnormally
    io:format("Child process exiting abnormally.~n"),
    exit(bad_event).

    % a non-normal exit signal is sent to parent, and since 
    % the parent is linked and not handling signals,
    % the parent ignores also dies 

