
-module(link5).
-export([start/0, child_process/0]).

% here we trap exit signals and now when the process gets some
% exit signal (normal or otherwise) the process decides how to 
% behave.
% it illustrates that a process does not HAVE to handle the exit
% signals, it simply keeps from ending when they happen
% by trapping them

start() ->
    % Parent process continues running after child finishes
    io:format("Parent>> process begins running.~n"),

    % Parent process sets the trap_exit flag to true to trap exit signals
    process_flag(trap_exit, true),

    % Parent process spawns and links to the child process
    ChildPid = spawn_link(?MODULE, child_process, []),
    io:format("Parent>> process has spawned and linked to child.~n"),

    % Receive and handle any exit signal from the child
    % if you want to but you dont have to

    % Parent process continues running after the abnormal exit signal
    % is sent and trapped... and ignored
    io:format("Parent process is still running.~n"),

    % Wait for a while before ending
    timer:sleep(2000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(2000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(3000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(3000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(3000),
    io:format("Parent process exiting normally.~n").



child_process() ->
    io:format("Child>> process is running.~n"),
    timer:sleep(2000),
    io:format("Child>> process is running.~n"),
    timer:sleep(2000),
    io:format("Child>> process is running.~n"),
    timer:sleep(2000),
    io:format("Child>> process is running.~n"),

    % Child process finishes abnormally
    io:format("Child process exiting abnormally.~n"),
    exit(bad_event).

    % a non-normal exit signal is sent to parent, and since 
    % the parent is linked and catching/handling signals,
    % the parent is able to keep from ending





