
-module(link6).
-export([start/0, child_process/0]).

% shows non-blocking handling of trapped signals
% here the receive block has a timer in it to 
% allow the parent to try to get signals, then move on
% to do other stuff


start() ->
    % Parent process sets the trap_exit flag to true to trap exit signals
    process_flag(trap_exit, true),

    % Parent spawns and links to the child process
    io:format("Parent spawns child .~n"),
    ChildPid = spawn_link(?MODULE, child_process, []),

    % Allow some time for the child to run
    % Simulate doing other work in the parent process
    timer:sleep(2000),
    io:format("Parent is doing other work while child runs.~n"),
    timer:sleep(1000),
    io:format("Parent is doing other work while child runs.~n"),
    timer:sleep(1000),
    io:format("Parent is doing other work while child runs.~n"),
    timer:sleep(1000),
    io:format("Parent is doing other work while child runs.~n"),
    
    % Continue to wait for exit signal, but do not block indefinitely
    receive
        {'EXIT', ChildPid, Reason} ->
            io:format("Parent received exit signal from child: ~p~n", [Reason])
    after 2000 ->  % Wait for 2 seconds before timing out
        io:format("Parent did not receive exit signal in time.~n")
    end,

    % Parent process continues running after receiving the 
    % abnormal exit signal or timing out
    io:format("Parent process still running after checking for exit signals.~n"),
    % Final sleep before exiting
    timer:sleep(2000),
    io:format("Parent process exiting normally.~n").



child_process() ->
    io:format("Child process is running.~n"),
    
    % Simulate some work in the child process
    timer:sleep(1000),
    io:format("Child process running.~n"),
    timer:sleep(1000),
    io:format("Child process running.~n"),
    timer:sleep(1000),
    io:format("Child process running.~n"),

    % Child process exits abnormally with a custom reason
    io:format("Child process executing abnormally.~n"),
    exit(bad_thing_happened).

