
-module(monex).
-export([start/0, worker_loop/0, monitor_loop/1]).

%% Start the monitoring process
start() ->
    %% Spawn the worker process
    WorkerPid = spawn(?MODULE, worker_loop, []),
    %% Spawn the monitor process
    spawn(?MODULE, monitor_loop, [WorkerPid]).

%% The worker process (this process will die unexpectedly)
worker_loop() ->
    io:format("Worker started with PID ~p~n", [self()]),
    receive
        %% Simulate some work and then crash
        crash ->
            io:format("Worker ~p crashing...~n", [self()]),
            exit(crash_reason);
        _Other ->
            io:format("Worker ~p continuing...~n", [self()]),
            worker_loop()
    end.

%% The monitor process
monitor_loop(WorkerPid) ->
    %% Set up a monitor to watch the worker process
    io:format("Monitor started, watching ~p~n", [WorkerPid]),
    Ref = erlang:monitor(process, WorkerPid),
    
    %% Wait for the worker to die or receive a message
    receive
        {'DOWN', Ref, process, _Pid, Reason} ->
            io:format("Worker ~p died. Reason: ~p~n", [WorkerPid, Reason]),
            %% Restart the worker process
            NewWorkerPid = spawn(?MODULE, worker_loop, []),
            %% Monitor the new worker process
            monitor_loop(NewWorkerPid);
        _Other ->
            io:format("Monitor received an unexpected message~n"),
            monitor_loop(WorkerPid)
    end.

